import z from 'zod';
import * as zod_to_json_schema from 'zod-to-json-schema';

type TActionMetadata = {
    /**
     * The public facing name of the action. This should be a verb phrase.
     *
     * @example "Create issue"
     * @example "Get availabilities"
     * @example "Get Google contact"
     */
    title: string;
    /**
     * A short description of what the action does.
     *
     * @example "Creates an issue in Linear"
     * @example "Gets the availabilities of a user using Google's FreeBusy API"
     * @example "Gets a contact from Google Contacts"
     */
    description: string;
    /**
     * The name of the resource that the action operates on.
     *
     * @example "Google Calendar"
     * @example "Gmail"
     * @example "Notion"
     */
    resource: string;
    /**
     * The default keywords the user will have to reference the action. Spaces are not allowed.
     *
     * So for example, if the keyword is "create-issue", the user will be able to type @ create-issue to reference the action.
     *
     * @example ["create-issue"]
     * @example ["get-google-contact"]
     * @example ["send-mail", "send-google-mail"]
     */
    defaultKeywords: Readonly<[string, ...string[]]>;
    /**
     * The image to display as the avatar/icon for the action. The `light` source is used when the light theme is enabled, and the `dark` source is used when the dark theme is enabled.
     *
     * @example
     * The Google logo is used for the `Google Calendar` action and is under the filepath of "/logos/google.svg".
     * ```
     * {
     *   light: "/logos/google.svg",
     *   dark: "/logos/google.svg"
     * }
     * ```
     *
     * @example
     * The Energize AI logo is used for the `feedback` action. We want the black version of the logo to be used in the light theme, and the white version of the logo to be used in the dark theme.
     * ```
     * {
     *   light: "/logos/energize-black.svg",
     *   dark: "/logos/energize-white.svg"
     * }
     * ```
     */
    avatar: Readonly<{
        /**
         * The image to display when the light theme is enabled.
         */
        light: string;
        /**
         * The image to display when the dark theme is enabled.
         */
        dark: string;
    }>;
    /**
     * The text to encode in the user message when the action is invoked.
     *
     * @example
     * When the user types @get-availabilities, the message "Please check my calendar for availability" is encoded in the user message.
     * ```
     * "Please check my calendar for availability"
     * ```
     *
     * @example
     * When the user types @get-google-contact, the message "Please get my Google contact" is encoded in the user message.
     * ```
     * "Please get my Google contact"
     * ```
     */
    chatMessage: string;
    /**
     * The URL to the API reference for the action. This is used to generate the "Learn more" button in the action details page.
     *
     * @example
     * The API reference for the `Google Calendar API` is used for the Google Calendar action.
     * ```
     * "https://developers.google.com/calendar/api/v3/reference"
     * ```
     *
     * @example
     * The API reference for the `Google People API` is used for the Google Contacts action.
     * ```
     * "https://developers.google.com/people/api/rest/v1/people/get"
     * ```
     */
    apiReference: string | null;
    /**
     * Useful examples to get the user started with the action. This is used to generate the "Examples" section in the action details page.
     *
     * @example
     * The following examples are used for the `Google Calendar` create event action.
     * ```
     * [
     *  "I have a dentist appointment on Monday at 2pm",
     *  "I have a meeting with John on Tuesday at 3pm",
     *  "I have a doctor's appointment on June 1st at 10am",
     * ]
     * ```
     */
    examples: [string, ...string[]] | null;
    /**
     * The loading message to stream in the chat when the action gets invoked.
     *
     * @example
     * The following loading message is used for the `Google Contacts` action.
     * ```
     * "Searching your contacts"
     * ```
     *
     * @example
     * The following loading message is used for the `DALLE-3` action.
     * ```
     * "Generating an image with DALLE-3"
     * ```
     */
    loadingMessage?: string;
    /**
     * When the LLM calls the tool, sometimes you want to give it data only available at runtime. We have found that user data and the local time zone are useful to have at runtime.
     *
     * @param localTimeZone The local time zone of the user. This is a string in the format of "America/Los_Angeles".
     * @param userData The user data of the user. This is an object with the properties `name` and `email`.
     *
     * @returns A string that will be used at runtime when the LLM calls the tool. If null is returned, the LLM will use the description of the input schema.
     *
     * @example
     * A useful thing to pass to the LLM when it has a tool that deals with dates are things like the current day of the week, the current time, and the local time zone.
     * Here is how you would pass that to the LLM.
     *
     * ```
     * ({ localTimeZone }) => {
     *   const { dayOfWeek, currentTimeRFC3339 } = getDayOfWeekAndTime();
     *   return `For reference, the current day is ${dayOfWeek}, the current time is ${currentTimeRFC3339}, the local timezone is ${localTimeZone}.`
     * }
     * ```
     *
     * @example
     * When sending emails, it is helpful for the LLM to know the user's name so it can personalize the email.
     *
     * ```
     * ({ userData }) => {
     *  return `The user's name is ${userData.name}, so make sure to use that in the email signature.`
     * }
     * ```
     */
    runTimeDescriptionGenerator?: (_: {
        localTimeZone: string;
        userData: {
            name: string;
            email: string;
        };
    }) => string | null;
};
declare class ActionMetadata {
    protected metadata: TActionMetadata;
    constructor({ title, description, resource, defaultKeywords, avatar, chatMessage, apiReference, loadingMessage, examples, runTimeDescriptionGenerator, }: Omit<TActionMetadata, "chatMessage" | "loadingMessage" | "examples" | "runTimeDescriptionGenerator"> & {
        chatMessage?: string;
        loadingMessage?: string;
        examples?: TActionMetadata["examples"];
        runTimeDescriptionGenerator?: TActionMetadata["runTimeDescriptionGenerator"];
    });
    getMetadata(): TActionMetadata;
}
declare const createActionMetadata: (input: ConstructorParameters<typeof ActionMetadata>[0]) => ActionMetadata;

/**
 * The type of authentication.
 */
declare const AuthType: {
    readonly NONE: "None";
    readonly TOKEN: "Token";
    readonly OAUTH: "OAuth";
};
type TAuthType = ValuesOf<typeof AuthType>;
/**
 * Base type for configuring authentication.
 */
type TAuthConfigBase = {
    /**
     * A configuration object for the button users will see before authenticating.
     */
    button: {
        /**
         * The text to display on the button.
         *
         * @example
         * The following text is used for the `Google Calendar` authentication button.
         * ```
         * "Continue with Google"
         * ```
         *
         * @example
         * The following text is used for the `Linear` authentication button.
         * ```
         * "Continue with Linear"
         * ```
         */
        text: string;
    };
    /**
     * The URL to the privacy policy for the authentication.
     *
     * @example
     * The following URL is used for the `Google People API` authentication.
     * ```
     * "https://developers.google.com/people/v1/getting-started"
     * ```
     */
    policyReferenceURL?: string;
    /**
     * A human readable name for the scope of the authentication.
     *
     * @example
     * The following name is used for the `Google Calendar` read scope authentication.
     * ```
     * "Google Calendar"
     * ```
     *
     * @example
     * The following name is used for the `Google Contacts` read scope authentication.
     * ```
     * "Google Contacts"
     * ```
     */
    humanReadableName: string;
    /**
     * A human readable description for the scope of the authentication.
     *
     * @example
     * The following description is used for the `Google Calendar` read scope authentication.
     * ```
     * "Read-only access to your Google Calendar"
     * ```
     *
     * @example
     * The following description is used for the `Google Calendar` write scope authentication.
     * ```
     * "Read and write access to your Google Calendar"
     * ```
     */
    humanReadableDescription: string;
};
/**
 * Configuration for OAuth-based authentication.
 */
type TOAuthConfig = TAuthConfigBase & {
    /**
     * An non-empty array of scopes required for the OAuth authentication.
     *
     * @example
     * The following scopes are used for the `Google Contacts` authentication.
     * ```
     * ["https://www.googleapis.com/auth/contacts.other.readonly", "https://www.googleapis.com/auth/contacts.readonly"]
     * ```
     */
    scopes: [string, ...string[]];
    /**
     * A URL that will take the user to where they can generate their own OAuth app to obtain a client ID and client secret.
     */
    oauthAppGenerationURL: string;
    /**
     * A URL to documentation that helps users understand the OAuth process or set it up.
     */
    documentationURL?: string;
} & ({
    /**
     * The URL of the OAuth 2.0 Discovery Endpoint.
     *
     * @example
     * For google, this is `https://accounts.google.com/.well-known/openid-configuration`.
     */
    discoveryEndpoint: string;
} | {
    discoveryEndpoint: undefined;
    /**
     * The URL of the authorization endpoint for OAuth.
     *
     * @example
     * For google, this is `https://accounts.google.com/o/oauth2/v2/auth`.
     */
    authorizationEndpoint: string;
    /**
     * The URL of the token endpoint for obtaining access and refresh tokens.
     *
     * @example
     * For google, this is `https://oauth2.googleapis.com/token`.
     */
    tokenEndpoint: string;
    /**
     * The URL of the refresh endpoint for refreshing access tokens.
     *
     * @example
     * For google, this is `https://oauth2.googleapis.com/token`.
     */
    refreshEndpoint?: string;
    /**
     * The URL of the revoke endpoint for revoking access tokens.
     *
     * @example
     * For google, this is `https://oauth2.googleapis.com/revoke`.
     */
    revokeEndpoint: string;
    /**
     * The code challenge method used for PKCE. It can either be 'S256' or null.
     */
    codeChallengeMethod: "S256" | null;
});
type TTokenCustomData = z.ZodObject<any> | null;
/**
 * Configuration for authenticating with an access token that users must generate themselves.
 *
 * This is useful for resources that don't support OAuth, but do support API keys.
 */
type TTokenAuthConfig<TCustomData extends TTokenCustomData> = TAuthConfigBase & {
    /**
     * The URL for documentation on how to generate an access token from the provider.
     *
     * This will be displayed for users when they are configuring the action to help them get setup.
     */
    generatingTokenReferenceURL: string;
    /**
     * Optional schema for custom data to be stored in the user's account that is needed to use the action.
     *
     * @example
     * For the Canvas LTS action, we also need to collect the user's Canvas domain (e.g. `https://canvas.calpoly.edu`).
     * ```
     * z.object({
     *  canvasDomain: z.string().url().refine((url) => url.startsWith("https://"), {
     *   message: "Canvas domain must start with https://",
     *  }),
     * })
     * ```
     */
    customDataSchema: TCustomData;
    /**
     * An optional function to validate the access token and custom data upon submission.
     *
     * @example
     * For the Canvas LTS action, we need to validate the access token and custom data by making a request to the Canvas API.
     *
     * ```typescript
     * validateToken: async ({ auth }) => {
     *  const response = await fetch(`${auth.customData.canvasDomain}/api/v1/users/self`, {
     *   headers: {
     *    Authorization: `Bearer ${auth.accessToken}`,
     *   },
     *  })
     *
     *  return { isValid: response.ok }
     * }
     * ```
     */
    validateToken?: (_: {
        auth: {
            customData: TCustomData extends z.ZodObject<any> ? z.infer<TCustomData> : null;
            accessToken: string;
        };
    }) => Promise<{
        isValid: boolean;
    }>;
};
/**
 * The authentication configuration for an action.
 */
type TActionAuth = {
    type: typeof AuthType.NONE;
    config: undefined;
} | {
    type: typeof AuthType.TOKEN;
    config: TTokenAuthConfig<any>;
} | {
    type: typeof AuthType.OAUTH;
    config: TOAuthConfig;
};
/**
 * Generic type for getting the auth argument in the action input function.
 *
 * The auth argument will always have an `accessToken` property, but it may also have a `customData` property if the auth config has a custom data schema.
 */
type TAuthArg<TAuth extends TActionAuth> = TAuth["config"] extends TTokenAuthConfig<any> ? {
    customData: TAuth["config"]["customDataSchema"] extends null ? null : z.infer<TAuth["config"]["customDataSchema"]>;
    accessToken: string;
} : TAuth["config"] extends TOAuthConfig ? {
    accessToken: string;
} : null;

type TActionFunction<TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> = (_: {
    input: z.infer<TSubmission extends undefined ? TInput : TSubmission>;
    auth: TAuthArg<TAuth>;
    userData: TActionUserData;
}) => Promise<z.infer<TOutput>>;
declare class ActionBuilderWithFunction<TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> {
    protected actionData: TActionData<TId, TInput, TOutput, TAuth, TSubmission>;
    constructor({ actionData, }: {
        actionData: TActionData<TId, TInput, TOutput, TAuth, TSubmission>;
    });
    setExampleInput(exampleInput: z.infer<TInput>): this;
    getId(): TId;
    getExampleInput(): z.TypeOf<TInput> | null;
    getMetadata(): {
        title: string;
        description: string;
        resource: string;
        defaultKeywords: readonly [string, ...string[]];
        avatar: Readonly<{
            light: string;
            dark: string;
        }>;
        chatMessage: string;
        apiReference: string | null;
        examples: [string, ...string[]] | null;
        loadingMessage?: string | undefined;
        runTimeDescriptionGenerator?: ((_: {
            localTimeZone: string;
            userData: {
                name: string;
                email: string;
            };
        }) => string | null) | undefined;
    };
    getAuthConfig(): TAuth;
    getInputSchema(): TInput;
    getSubmissionSchema(): TSubmission | undefined;
    getActionFunction(): TActionFunction<TInput, TOutput, TAuth, TSubmission>;
    getOutputSchema(): TOutput;
    getInputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getOutputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getComponent(): TPassThroughComponent<TInput, TOutput, TSubmission>;
}

type TActionBuilderWithAuthData<TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> = TActionBuilderWithOutputData<TId, TInput, TOutput, TSubmission> & Pick<TActionData<TId, TInput, TOutput, TAuth, TSubmission>, "authConfig">;
declare class ActionBuilderWithAuth<TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> {
    protected actionData: TActionBuilderWithAuthData<TId, TInput, TOutput, TAuth, TSubmission>;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithAuthData<TId, TInput, TOutput, TAuth, TSubmission>;
    });
    /**
     * The function that gets called when the action is invoked. This function should return a promise that resolves to the output of the action.
     *
     * @param input The input to the action function. This is the input that the user provides when invoking the action.
     * @param auth The authentication information for the action. This is the authentication information that the user provides when invoking the action.
     *
     * @returns A promise that resolves to the output of the action.
     *
     * @example
     * The following action function is an example for a `send-email` action.
     * ```
     * async ({ input, auth }) => {
     *   const { subject, body, to } = input;
     *   const { accessToken } = auth;
     *   ...code to send email using the input...
     *   return;
     * }
     * ```
     *
     * @example
     * The following action function is an example for a `get-google-contact` action.
     * ```
     * async ({ input, auth }) => {
     *   const contact = ...code to get contact...
     *   return contact;
     * }
     * ```
     */
    setActionFunction(actionFunction: TActionFunction<TInput, TOutput, TAuth, TSubmission>): ActionBuilderWithFunction<TId, TInput, TOutput, TAuth, TSubmission>;
}

declare class ActionBuilderWithOAuthType<TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit = undefined> {
    protected actionData: TActionBuilderWithOutputData<TId, TInput, TOutput, TSubmission>;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithOutputData<TId, TInput, TOutput, TSubmission>;
    });
    setOAuthData: (data: TOAuthConfig) => ActionBuilderWithAuth<TId, TInput, TOutput, {
        type: "OAuth";
        config: TOAuthConfig;
    }, TSubmission>;
}

declare class ActionBuilderWithTokenType<TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit = undefined> {
    private actionData;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithOutputData<TId, TInput, TOutput, TSubmission>;
    });
    setTokenData: <T extends TTokenCustomData>(data: TTokenAuthConfig<T>) => ActionBuilderWithAuth<TId, TInput, TOutput, {
        type: "Token";
        config: TTokenAuthConfig<T>;
    }, TSubmission>;
}

type TNoAuth = {
    type: typeof AuthType.NONE;
    config: undefined;
};
type TActionBuilderWithOutputData<TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit = undefined> = TActionBuilderWithInputData<TId, TInput> & Pick<TActionData<TId, TInput, TOutput, any, TSubmission>, "outputSchema" | "component" | "submissionSchema">;
declare class ActionBuilderWithOutput<TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit = undefined> {
    private actionData;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithOutputData<TId, TInput, TOutput, TSubmission>;
    });
    /**
     * The authentication type for the action. This is used to generate the authentication page for the action and get users authenticated.
     */
    setAuthType(type: typeof AuthType.NONE): ActionBuilderWithAuth<TId, TInput, TOutput, TNoAuth, TSubmission>;
    setAuthType(type: typeof AuthType.TOKEN): ActionBuilderWithTokenType<TId, TInput, TOutput, TSubmission>;
    setAuthType(type: typeof AuthType.OAUTH): ActionBuilderWithOAuthType<TId, TInput, TOutput, TSubmission>;
}

type TActionComponentPropsData<TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit = undefined> = {
    /**
     * The input to the action function. You can use this input to prepopulate the form so all the user has to do is click submit.
     */
    input: z.infer<TInput>;
    /**
     * The user data for the action. This is the data that the user has provided to the LLM. You can use this data to prepopulate the form so all the user has to do is click submit.
     */
    userData: TActionUserData;
    /**
     * Indicates whether the action function is currently running.
     */
    isLoading: boolean;
    /**
     * Indicates whether the action function has completed running and is successful.
     */
    isSuccess: boolean;
    /**
     * Indicates whether the action function has completed running and has failed.
     */
    isError: boolean;
    /**
     * The submit handler for the form. This is called when the user clicks the submit button.
     */
    onSubmit: (formValues: z.infer<TOnSubmitValues extends undefined ? TInput : TOnSubmitValues>) => Promise<void>;
};
type TActionComponentProps<TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit = undefined> = {
    inputSchema: TInput;
    metadata: ReturnType<ActionMetadata["getMetadata"]>;
} & ({
    /**
     *  When the action is active, the user can interact with the component.
     */
    displayState: "active";
    data: TActionComponentPropsData<TInput, TOnSubmitValues>;
} | {
    /**
     * A placeholder component is used for things like the action details page, where the user can view the component but not interact with it.
     */
    displayState: "placeholder";
    data: undefined;
} | {
    /**
     * Indicates whether the `LLM is still streaming in the input to the action function`.
     * If this is the case, it is necessary to show a loading state of the form.
     * We recommend using Skeleton components instead of input fields to indicate that the form is loading.
     */
    displayState: "skeleton";
    data: undefined;
});
type TActionComponent<TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit = undefined> = React.FC<TActionComponentProps<TInput, TOnSubmitValues>>;
type TPassThroughComponent<TInput extends TActionInput, TOutput extends TActionOutput, TOnSubmitValues extends TActionOnSubmit = undefined> = TOutput extends z.ZodVoid ? TActionComponent<TInput, TOnSubmitValues> : null;
/**
 * For actions that do not return any data (i.e. POST actions), you need to specify a component to render in the chat after the action is invoked.
 * This component should be a form asking for confirmation to submit the input to the action function.
 */
declare class ActionBuilderWithPost<TId extends string, TInput extends TActionInput, TSubmission extends TActionOnSubmit = undefined> {
    private actionData;
    private submissionSchema?;
    constructor({ actionData, submissionSchema, }: {
        actionData: TActionBuilderWithInputData<TId, TInput>;
        submissionSchema?: TSubmission;
    });
    setComponentSubmissionSchema<T extends TActionInput>(submissionSchema: T): ActionBuilderWithPost<TId, TInput, T>;
    setOutputComponent(component: React.FC<TActionComponentProps<TInput, TSubmission>>): ActionBuilderWithOutput<TId, TInput, z.ZodVoid, TSubmission>;
}

type TActionData<TId extends string, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> = {
    id: TId;
    metadata: ActionMetadata;
    inputSchema: TInput;
    submissionSchema?: TSubmission;
    outputSchema: TOutput;
    authConfig: TAuth;
    actionFunction: TActionFunction<TInput, TOutput, TAuth, TSubmission>;
    component: TPassThroughComponent<TInput, TOutput, TSubmission>;
    exampleInput: z.infer<TInput> | null;
};

declare class ActionBuilderWithGet<TId extends string, TInput extends TActionInput> {
    private actionData;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithInputData<TId, TInput>;
    });
    /**
     * `JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     */
    setOutputSchema(output: z.ZodObject<any>): ActionBuilderWithOutput<TId, TInput, z.ZodObject<any, z.UnknownKeysParam, z.ZodTypeAny, {
        [x: string]: any;
    }, {
        [x: string]: any;
    }>, undefined>;
}

type TActionBuilderWithInputData<TId extends string, TInput extends TActionInput> = Pick<TActionData<TId, TInput, any, any, any>, "metadata" | "inputSchema" | "id">;
declare class ActionBuilderWithInput<TId extends string, TInput extends TActionInput> {
    private actionData;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithInputData<TId, TInput>;
    });
    /**
     * Spark has two options for actions:
     *
     * 1.`JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     *
     * 2.`React Component` that asks for confirmation for the action. This is typically the output for actions that `POST data`. If this is the case, you should specify a React component that asks for confirmation for the action. The output will be `void`.
     */
    setActionType(type: "GET"): ActionBuilderWithGet<TId, TInput>;
    setActionType(type: "POST"): ActionBuilderWithPost<TId, TInput>;
}

type TActionInput = z.ZodObject<any>;
type TActionOutput = z.ZodObject<any> | z.ZodVoid;
type TActionOnSubmit = z.ZodObject<any> | undefined;
declare const ActionUserDataSchema: z.ZodObject<{
    email: z.ZodString;
    name: z.ZodString;
}, "strip", z.ZodTypeAny, {
    email: string;
    name: string;
}, {
    email: string;
    name: string;
}>;
type TActionUserData = z.infer<typeof ActionUserDataSchema>;
declare class ActionBuilder<TId extends string> {
    private metadata;
    private id;
    constructor(input: {
        metadata: ActionMetadata;
        id: TId;
    });
    /**
     * The Zod input schema for the action. This is used to validate the arguments passed to the action.
     *
     * You must include a `.describe()` call on each field to provide a description of the input. This description will be used to generate the prompts and details page for your action.
     *
     * @example
     * The following input schema is an example for a `send-email` action.
     * ```
     * z.object({
     *  subject: z.string().desribe("The subject of the email"),
     *  body: z.string().describe("The body of the email"),
     *  to: z.array(z.object({
     *   email: z.string().describe("The email of the recipient"),
     * })).describe("The recipients of the email"),
     * ```
     */
    setInputSchema<T extends TActionInput>(input: T): ActionBuilderWithInput<TId, T>;
}
/**
 * Create a new action.
 */
declare const createAction: <TId extends string>(input: {
    /**
     * The metadata for the action. This includes the title, description, resource, and other details about the action.
     */
    metadata: ActionMetadata;
    /**
     * The unique identifier for the action. This is used to reference the action in the registry.
     *
     * Guidelines for Creating IDs:
     * Prefix with Service: Start with a prefix that identifies the service (e.g., Google, Bing).
     * Action Description: Follow the service prefix with a short, descriptive action name.
     * Use Hyphens: Separate words with hyphens (e.g., google-searchEmailInbox).
     * Use Camel Case: Use camel case for action names (e.g., google-searchEmailInbox).
     *
     * @example
     * ```
     * id: "google-searchEmailInbox"
     * ```
     */
    id: TId;
}) => ActionBuilder<TId>;

type TActionsRegistry<T extends ActionBuilderWithFunction<any, any, any, any, any>[]> = T extends [infer A, ...infer R] ? A extends ActionBuilderWithFunction<infer Id, any, any, any, any> ? R extends ActionBuilderWithFunction<string, any, any, any, any>[] ? Readonly<{
    [K in Id]: A;
}> & TActionsRegistry<R> : never : never : Readonly<{}>;
/**
 * Create a registry of actions.
 *
 * @param registry The list of actions to register.
 * @returns The registry of actions.
 */
declare const createActionsRegistry: <const T extends ActionBuilderWithFunction<any, any, any, any, any>[]>(registry: T) => TActionsRegistry<T>;
/**
 * Filter the action registry by the auth type.
 *
 * @param registry The registry of actions.
 * @param authType The auth type to filter by.
 *
 * @returns The filtered registry of actions.
 */
declare const filterActionRegistryByAuthType: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any, any, any, any, any>;
}>, U extends TAuthType>(registry: T, authType: U) => { [K in keyof T as ReturnType<T[K]["getAuthConfig"]>["type"] extends U ? K : never]: T[K]; };

/**
 * Defines a structure for 'ScopePoolOr', which represents a logical OR condition in a scope pool.
 *
 * @example
 * This example signifies that the user must have both the "read:issue" and "read:organization" scopes.
 * ```
 * {
 *   and: ["read:issue", "read:organization"]
 * }
 * ```
 */
type TScopePoolNode = {
    and: [string, ...string[]];
};
/**
 * The scope pool that the user must have in order to use the action.
 *
 * @example
 * To read events from Google Calendar, the user must have either the readonly scope or the general events scope.
 * ```
 * {
 *   or: [
 *     {
 *       and: ["https://www.googleapis.com/auth/calendar.events.readonly"]
 *     },
 *     {
 *       and: ["https://www.googleapis.com/auth/calendar.events"]
 *     }
 *   ]
 * }
 * ```
 *
 * @example
 * The get weather API does not required any authentication, so the scope pool is null.
 * ```
 * null
 * ```
 */
type TScopePool = {
    or: [TScopePoolNode, ...TScopePoolNode[]];
};

type ValuesOf<T> = T[keyof T];

export { ActionBuilder, ActionBuilderWithFunction, ActionMetadata, ActionUserDataSchema, AuthType, type TActionAuth, type TActionComponent, type TActionFunction, type TActionInput, type TActionOnSubmit, type TActionOutput, type TActionUserData, type TAuthArg, type TAuthConfigBase, type TAuthType, type TOAuthConfig, type TScopePool, type TTokenAuthConfig, type TTokenCustomData, type ValuesOf, createAction, createActionMetadata, createActionsRegistry, filterActionRegistryByAuthType };
